from typing import Dict, List, TypeVar, Union

from grafanarmadillo.types import DashboardContent, DashboardSearchResult


A = TypeVar("A")


def flat_map(f, xs):
	"""
	Flatmap: Map on a list and then merge the results.
	
	>>> and_reversed = lambda s: [s,s[::-1]]

	>>> flat_map(and_reversed, [])
	[]

	>>> flat_map(and_reversed, ['hi'])
	['hi', 'ih']

	>>> flat_map(and_reversed, ['hi', 'hello'])
	['hi', 'ih', 'hello', 'olleh']

	"""
	ys = []
	for x in xs:
		ys.extend(f(x))
	return ys


def exactly_one(items: List[A], msg="did not find exactly one item") -> A:
	"""
	Throws if list does not contain exactly 1 item.
	
	>>> exactly_one([1])
	1

	>>> exactly_one([1,2])
	Traceback (most recent call last):
	ValueError: did not find exactly one item

	>>> exactly_one([])
	Traceback (most recent call last):
	ValueError: did not find exactly one item

	"""
	if len(items) != 1:
		raise ValueError(msg)
	return items[0]


def project_dict(d: Dict, keys: set) -> Dict:
	"""
	Select the given fields from a dictionary.
	
	>>> project_dict({'a': 1, 'b': 2}, set(['a']))
	{'a': 1}
	"""
	return {k: v for k, v in d.items() if k in keys}


def project_dashboard_identity(
	dashboardlike: Union[DashboardSearchResult, DashboardContent]
) -> Dict:
	"""Project only the fields of a dashboard which are used for determining identity."""
	meta_fields = set(["id", "uid", "title"])
	return project_dict(dashboardlike, meta_fields)



"""Find Grafana dashboards and folders."""

from pathlib import PurePath
from typing import List, Tuple

from grafana_api.grafana_face import GrafanaFace

from grafanarmadillo._util import exactly_one, flat_map
from grafanarmadillo.types import DashboardSearchResult, FolderSearchResult


class Finder(object):
	"""Collection of methods for finding Grafana dashboards and folders."""

	def __init__(self, api: GrafanaFace) -> None:
		super().__init__()
		self.api = api

	def find_dashboards(self, name) -> List[DashboardSearchResult]:
		"""Find all dashboards with a name. Returns exact matches only."""
		return list(
			filter(
				lambda x: x["title"] == name,
				self.api.search.search_dashboards(query=name, type_="dash-db"),
			)
		)

	def _enumerate_dashboards_in_folders(self, folder_ids: List[str]):
		folder_param = ",".join(folder_ids)
		return self.api.search.search_dashboards(
			query=None, type_="dash-db", folder_ids=folder_param
		)

	def get_dashboards_in_folders(
		self, folder_names: List[str]
	) -> List[DashboardSearchResult]:
		"""Get all dashboards in folders."""
		folder_objects = list(
			map(lambda folder_name: self.get_folder(name=folder_name), folder_names)
		)
		return self._enumerate_dashboards_in_folders(
			list(map(lambda f: str(f["id"]), folder_objects))
		)

	def get_folder(self, name) -> FolderSearchResult:
		"""Get a folder by name. Folders don't nest, so this will return at most 1 folder."""
		if name == "General":
			return self.api.folder.get_folder_by_id(0)
		else:
			search_result = self.api.search.search_dashboards(query=name, type_="dash-folder")
			return exactly_one(
				list(
					filter(
						lambda x: x["title"] == name,
						map(lambda sr: self.api.folder.get_folder(sr["uid"]), search_result),
					)
				)
			)

	def get_dashboard(self, folder_name, dashboard_name) -> DashboardSearchResult:
		"""
		Get a dashboard by its parent folder and dashboard name.

		Dashboards without a parent are children of the "General" folder.
		"""
		folder_object = self.get_folder(folder_name)
		dashboards = self._enumerate_dashboards_in_folders([str(folder_object["id"])])
		return exactly_one(list(filter(lambda d: d["title"] == dashboard_name, dashboards)))

	def _resolve_path(self, path) -> Tuple[str, str]:
		parts = PurePath(path).parts

		# validate path
		if len(parts) > 3:
			# can support maximally ("/", "folder", "dashboard")
			raise ValueError("Dashboard path has too many parts")
		if PurePath(path).is_absolute() or parts[0] == "/" or parts[0] == "\\":
			# removes optional "absolurte" signifier
			parts = parts[1:]

		if len(parts) == 2:
			folder, dashboard = parts
		else:
			folder, dashboard = "General", parts[0]

		return folder, dashboard

	def get_from_path(self, path) -> DashboardSearchResult:
		"""Get a dashboard from a string path like `/folder0/dashboard0`."""
		folder, dashboard = self._resolve_path(path)
		return self.get_dashboard(folder, dashboard)



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""Setup dot py."""
from __future__ import absolute_import, print_function

# import re
from glob import glob
from os.path import basename, dirname, join, splitext

from setuptools import find_packages, setup


def read(*names, **kwargs):
	"""Read description files."""
	path = join(dirname(__file__), *names)
	with open(path, encoding=kwargs.get("encoding", "utf8")) as fh:
		return fh.read()


# previous approach used to ignored badges in PyPI long description
# long_description = '{}\n{}'.format(
#     re.compile(
#         '^.. start-badges.*^.. end-badges',
#         re.M | re.S,
#         ).sub(
#             '',
#             read('README.rst'),
#             ),
#     re.sub(':[a-z]+:`~?(.*?)`', r'``\1``', read(join('docs', 'CHANGELOG.rst')))
#     )

long_description = "{}\n{}".format(
	read("README.rst"), read(join("docs", "CHANGELOG.rst")),
)

setup(
	name="grafanarmadillo",
	version="0.0.6",
	description="Simplifies interacting with Grafana, with a focus on templating dashboards ",
	long_description=long_description,
	long_description_content_type="text/x-rst",
	license="MIT License",
	author="lilatomic",
	url="https://github.com/lilatomic/grafanarmadillo",
	packages=find_packages("src"),
	package_dir={"": "src"},
	py_modules=[splitext(basename(i))[0] for i in glob("src/*.py")],
	include_package_data=True,
	zip_safe=False,
	classifiers=[],
	project_urls={
		"webpage": "https://github.com/lilatomic/grafanarmadillo",
		"Documentation": "https://grafanarmadillo.readthedocs.io/en/latest/",
		"Changelog": "https://github.com/lilatomic/grafanarmadillo/blob/main/docs/CHANGELOG.rst",
		"Issue Tracker": "https://github.com/lilatomic/grafanarmadillo/issues",
		"Discussion Forum": "https://github.com/lilatomic/grafanarmadillo/discussions",
	},
	keywords=[],
	python_requires=">=3.6, <3.9",
	install_requires=["grafana_api~=1.0"],
	extras_require={
		# eg:
		#   'rst': ['docutils>=0.11'],
		#   ':python_version=="2.6"': ['argparse'],
	},
	setup_requires=[
		#   'pytest-runner',
		#   'setuptools_scm>=3.3.1',
	],
	entry_points={
		"console_scripts": []
		#
	},
	# cmdclass={'build_ext': optional_build_ext},
	# ext_modules=[
	#    Extension(
	#        splitext(relpath(path, 'src').replace(os.sep, '.'))[0],
	#        sources=[path],
	#        include_dirs=[dirname(path)]
	#    )
	#    for root, _, _ in os.walk('src')
	#    for path in glob(join(root, '*.c'))
	# ],
)



"""Type hints for Grafana interaction."""

from typing import NewType


Dashboard = NewType("Dashboard", dict)
DashboardSearchResult = NewType("DashboardSearchResult", dict)
DashboardContent = NewType("DashboardContent", dict)
FolderSearchResult = NewType("Folder", dict)
